ion-icon {
    pointer-events: none;
}

.brick {
    margin-bottom: 1em;
}

.brick label {
    display: inline-block;
    width: 10em !important;
    vertical-align: top;
}

.bricks {
    display: flex;
}

.bricks .input.is-small {
    width: min-content;
}

button.is-loading {
    pointer-events: none;
    position: relative;
}

button.is-loading * {
    visibility: hidden;
}

button.is-loading::after {
    animation: spinAround 0.5s infinite linear;
    content: ".";
    display: block;
    height: 1em;
    position: absolute;
    top: 0;
}

.easel {
    position: relative;
    overflow-y: scroll;
    scrollbar-width: 0;
}

.easel::-webkit-scrollbar {
    display: none;
}

.svgi {
    filter: invert(0.6) sepia(1) saturate(1) hue-rotate(175deg);
}

.svgxs {
    width: 0.9em;
}

.is-active .svgi {
    filter: none;
}

tbody {
    display: block;
    height: 60vh;
    overflow-y: scroll;
}

thead,
tfoot,
tbody tr {
    display: table;
    width: 100%;
    table-layout: fixed;
}

.table {
    display: block;
}

.tabs ul li.is-active {
    border-bottom: 3px solid salmon !important;
    background-color: #e5eff9;
    font-weight: 900;
}

/* custom css */

.align-items-center {
    align-items: center;
}

.align-self-center {
    align-self: center;
}

.bg-light {
    background-color: #f9f9f9;
}

.flex-direction-column {
    flex-direction: column;
}

.flex-direction-row {
    flex-direction: row;
}

.font-800 {
    font-weight: 800;
}

.font-900 {
    font-weight: 900 !important;
}

.font-size-xs {
    font-size: 0.7em;
}

.font-size-sm {
    font-size: 1em;
}

.font-size-md {
    font-size: 1.6em;
}

.justify-content-center {
    justify-content: center;
}

.justify-content-space-around {
    justify-content: space-around;
}

.justify-content-space-between {
    justify-content: space-between;
}

.h-3em {
    height: 3.1em;
}

.h-100 {
    height: 100% !important;
}

.text-align-center {
    text-align: center;
}

.w-20em {
    width: 20em;
}

.w-30em {
    width: 30em;
}

.mw-40em {
    min-width: 40em;
}

.width-5em,
.w-5em {
    width: 5em;
}
